/*
 * Copyright (c) 2006, Your Corporation. All Rights Reserved.
 */

package org.jdesktop.jdic.screensaver;

import java.awt.*;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Base class for screensavers written in Java.
 * All developers should extend either <code>SimpleScreensaver</code>
 * or <code>JOGLScreensaver</code>.  This base class makes it possible to
 * offer different types of screensavers in the future, such as those that
 * can control their own rendering frequency.
 *
 * @author Mark Roth
 * @see SimpleScreensaver
 */
public abstract class ScreensaverBase {

    /**
     * Logging support
     */
    private static Logger logger =
            Logger.getLogger("org.jdesktop.jdic.screensaver");

    /**
     * Context information for this screensaver (eg window size)
     */
    protected ScreensaverContext context;

    /**
     * Last dimensions - if they change we need to clear the screen
     */
    private Dimension lastSize;

    /**
     * Called from native code to do initialization.  Subclasses should
     * override the init() method to provide any initialization.
     *
     * @param context Context information for this screensaver (eg window size)
     */
    public final void baseInit(ScreensaverContext context) {
        logger.log(Level.FINEST, "Screensaver starting up.");
        this.context = context;
        reinit();
    }

    /**
     * Called from native code to destroy the screensaver.  Subclasses
     * should override the destroy() method to provide any cleanup behavior.
     */
    public final void baseDestroy() {
        logger.log(Level.FINEST, "Screensaver shutting down.");
        try {
            destroy();
        }
        catch (Throwable t) {
            logger.log(Level.WARNING,
                    "Exception occurred during screensaver destroy()", t);
        }
    }

    /**
     * Gets the ScreenSaver context object
     *
     * @return The context object for this screen saver.
     */
    public ScreensaverContext getContext() {
        return context;
    }

    /**
     * Called from the native layer to render the next frame
     */
    public void renderFrame() {
        Component c = context.getComponent();
        if ((lastSize == null) ||
                (c.getWidth() != lastSize.width) ||
                (c.getHeight() != lastSize.height)) {
            lastSize = new Dimension(c.getWidth(), c.getHeight());
            reinit();
        }
    }

    /**
     * Initialize or reinitialize the screensaver.  Calls subclass init().
     * Any exceptions thrown during init() will be sent to the
     * org.jdesktop.jdic.screensaver J2SE logger.
     */
    private void reinit() {
        try {
            init();
        }
        catch (Throwable t) {
            logger.log(Level.WARNING,
                    "Exception occurred during screensaver init()", t);
        }
    }

    /**
     * Subclasses can optionally override this method to perform any
     * initialization.  Default does nothing.
     * <p/>
     * Init will be called once at the beginning, before the first paint,
     * and zero or more times during the running to reset the screensaver
     * (for example if the resolution changed).
     */
    protected void init() {
    }

    /**
     * Subclasses can optionally override this method to perform any
     * cleanup before the screensaver is destroyed.  Default does nothing.
     * <p/>
     * Destroy will be called at most once.  After a call to this
     * method, the screensaver should not expect the ScreensaverContext
     * to be valid any longer.
     */
    protected void destroy() {
    }

}
